//
//  AppController.m
//  ImageSorter
//
//  Created by Hjalti Jakobsson on 21.6.2009.
//  Copyright 2009 DP Software. All rights reserved.
//

#import "AppController.h"
#import "CalculateBrightnessOperation.h"

#define kCopyFiles 0

@implementation AppController
@synthesize results;

- (id)init
{
	if(self = [super init])
	{
		results = [[NSMutableArray alloc] init];
		queue = [[NSOperationQueue alloc] init];
		[queue setMaxConcurrentOperationCount:4];
	}
	
	return self;
}

- (void)dealloc
{
	[queue release];
	[results release];
	[super dealloc];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{	
	if([keyPath isEqualToString:@"operations"])
	{
		if([[queue operations] count] == 0)
		{
			[queue removeObserver:self forKeyPath:@"operations"];
			[self performSelectorOnMainThread:@selector(didFinishAllCalculation) withObject:nil waitUntilDone:NO];
		}
	}
	else
	{
		[super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
	}
}

- (IBAction)openFile:(id)sender
{
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	[openPanel setAllowsMultipleSelection:YES];
	
	if(([openPanel runModalForTypes:[NSBitmapImageRep imageFileTypes]] == NSOKButton))
	{
		[indicator startAnimation:nil];
		
		for(NSString *file in [openPanel filenames])
		{
			CalculateBrightnessOperation *op = [[CalculateBrightnessOperation alloc] initWithFilename:file delegate:self];
			[queue addOperation:op];
			[op release];
		}
		
		[queue addObserver:self forKeyPath:@"operations" options:0 context:NULL];
	}
}

- (void)didStartCalculating:(NSString*)file
{
	[label setStringValue:[NSString stringWithFormat:@"Calculating '%@'...", [file lastPathComponent]]];
}

- (void)copyFiles
{
	NSFileManager *manager = [NSFileManager defaultManager];

	for(NSDictionary *dict in results)
	{
		NSString *filename = [dict valueForKey:@"filename"];
		
		if([manager fileExistsAtPath:filename])
		{		
			NSNumber *value = [dict valueForKey:@"value"];		
			
			NSString *newFilename = [NSString stringWithFormat:@"%f.%@", [value floatValue], [filename pathExtension]];
			NSString *path = [filename stringByDeletingLastPathComponent];
			path = [path stringByAppendingPathComponent:newFilename];
			
			[manager copyPath:filename toPath:path handler:nil];
		}
	}
}

- (void)didFinishAllCalculation
{
	[label setStringValue:@""];

	[indicator stopAnimation:nil];
	
	if(kCopyFiles)
		[self copyFiles];
}

- (void)didFinishCalculating:(NSDictionary*)res
{	
	[self willChangeValueForKey:@"results"];
	
	[results addObject:res];
	
	[self didChangeValueForKey:@"results"];
}

@end
