//
//  CalculateBrightnessOperation.m
//  ImageSorter
//
//  Created by Hjalti Jakobsson on 22.6.2009.
//  Copyright 2009 DP Software. All rights reserved.
//

#import "CalculateBrightnessOperation.h"


@implementation CalculateBrightnessOperation
@synthesize filename, delegate;

- (id)initWithFilename:(NSString*)file delegate:(id)del
{
	if(self = [super init])
	{
		self.filename = file;
		self.delegate = del;
	}
	
	return self;
}

- (void)main
{	
	NSImage *image = [[NSImage alloc] initWithContentsOfFile:filename];
	
	NSBitmapImageRep *imageRep = [NSBitmapImageRep imageRepWithData:[image TIFFRepresentation]];
	
	NSInteger h = [imageRep pixelsHigh], w = [imageRep pixelsWide];
	unsigned char *data = [imageRep bitmapData];
	
	unsigned int samplesPerPixel = [imageRep samplesPerPixel];
	unsigned int sampleCount = [imageRep pixelsWide] * [imageRep pixelsHigh] * samplesPerPixel;
	
	unsigned int r = 0;
	unsigned int g = 1;
	unsigned int b = 2;
	
	float overall = 0;
	
	for (r, g, b ; b < sampleCount; r += samplesPerPixel, g += samplesPerPixel, b += samplesPerPixel)
	{
		float p = (data[r]/255.0+data[g]/255.0+data[b]/255.0)/3.0;
		overall += p;
	}
	
	overall = overall/(w*h);
	
	[image release];
	
	NSDictionary *result = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithFloat:overall], @"value", filename, @"filename", nil];
	
	if([delegate respondsToSelector:@selector(didFinishCalculating:)])
		[delegate performSelectorOnMainThread:@selector(didFinishCalculating:) withObject:result waitUntilDone:NO];
}

- (void)dealloc
{
	[filename release];
	[super dealloc];
}

@end
